package com.mindoo.ftp.internal;

import java.util.logging.Level;

import org.eclipse.core.runtime.Plugin;
import org.eclipse.osgi.framework.console.CommandProvider;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;

import com.mindoo.ftp.controller.FTPController;
import com.mindoo.ftp.logging.LogNotesThread;

/**
 * The activator class controls the plug-in life cycle
 */
public class FTPActivator extends Plugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "com.mindoo.ftp";

	// The shared instance
	private static FTPActivator plugin;
	private ServiceRegistration m_ftpCommandReg;
	private LogNotesThread m_logThread;

	/**
	 * The constructor
	 */
	public FTPActivator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugins#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		m_logThread=new LogNotesThread();
		m_logThread.start();

		m_ftpCommandReg=context.registerService(CommandProvider.class.getName(), new FTPCommandProvider(), null);

		Thread t=new Thread(){
			@Override
			public void run() {
				try {
					FTPController.startServer();
				}
				catch (Exception e) {
					FTPActivator.log(Level.SEVERE, "FTP startup failed", e);
				}
			}
		};
		t.start();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		m_ftpCommandReg.unregister();
		m_ftpCommandReg=null;
		
		FTPController.stopServer();
		FTPController.dispose();
		
		if (m_logThread!=null) {
			m_logThread.shutdown();
			m_logThread=null;
		}
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static FTPActivator getDefault() {
		return plugin;
	}

	/**
	 * Method to write a log entry. The entry is written to a
	 * log database if it can be found. Otherwise, we write to
	 * the server console.
	 * 
	 * @param level level
	 * @param message message
	 * @param t exception or <code>null</code>
	 */
	public static void log(Level level, String message, Throwable t) {
		FTPActivator.getDefault().m_logThread.log(level, message, t);
	}
}
